@@ -33,6 +33,10 @@ c_net_layout <- function(go, method = igraph::nicely(), order_by = NULL, order_l
   name <- x <- y <- NULL
   if ("igraph_layout_spec" %in% class(method)) {
     coors <- igraph::layout_(go, method)
+    if (all(is.na(coors))) {
+      go2 <- clean_igraph(go)
+      coors <- igraph::layout_(go2, method)
+    }
   } else if ("poly" %in% class(method)) {
     coors <- method(go, group2 = order_by, group2_order = order_ls)
   } else if ("layout" %in% class(method)) {
@@ -76,6 +80,18 @@ c_net_layout <- function(go, method = igraph::nicely(), order_by = NULL, order_l
   return(coors)
 }
 
+order_circle_func <- function(coors_mat) {
+  dis <- X <- Y <- NULL
+  mid_x <- mean(coors_mat[, 1])
+  mid_y <- mean(coors_mat[, 2])
+  coors_mat <- data.frame(X = coors_mat[, 1], Y = coors_mat[, 2]) %>%
+    dplyr::mutate(dis = sqrt((X - mid_x)^2 + (Y - mid_y)^2)) %>%
+    dplyr::arrange(dis) %>%
+    dplyr::select_at(1:2) %>%
+    as.matrix()
+  coors_mat
+}
+
 order_tmp_v_name <- function(tmp_v, coors_mat, order_by = NULL, order_ls = NULL) {
   if (is.null(order_by)) {
     coors <- data.frame(name = tmp_v$name, X = coors_mat[, 1], Y = coors_mat[, 2], row.names = NULL)
@@ -147,6 +163,19 @@ get_coors <- \(coors, go, ...){
   if (is.matrix(coors)) coors <- data.frame(name = V(go)$name, X = coors[, 1], Y = coors[, 2], row.names = NULL)
   # 5.如果是df了，那就对齐name用于下一步的绘图，
   if (is.data.frame(coors)) {
+    coors <- data.frame(coors, check.names = FALSE)
+    setdiff(V(go)$name, coors$name) -> tmp_name
+    if (length(setdiff(V(go)$name, coors$name)) > 0) {
+      warning("Some nodes are not in coors, will be randomly assigned")
+      coors <- rbind(
+        coors,
+        data.frame(
+          name = tmp_name,
+          X = runif(length(tmp_name), range(coors$X)[1], range(coors$X)[2]),
+          Y = runif(length(tmp_name), range(coors$Y)[1], range(coors$Y)[2])
+        )
+      )
+    }
     coors <- coors[match(V(go)$name, coors$name), ]
     coors <- as_coors(coors, edge_curved)
     return(coors)
@@ -156,6 +185,9 @@ get_coors <- \(coors, go, ...){
 
 rescale_coors <- function(coors, keep_asp = TRUE) {
   stopifnot(inherits(coors, "coors"))
+  if (any(is.na(coors$X)) || any(is.na(coors$Y))) {
+    stop("Some coordinates are NA")
+  }
   if (keep_asp) {
     diff_x <- diff(range(coors$X))
     diff_y <- diff(range(coors$Y))
@@ -216,6 +248,12 @@ transform_coors <- function(coors, scale = 1, aspect_ratio = 1,
   # 复制原始数据
   new_coor <- coors
 
+  mid_x <- mean(new_coor$X)
+  mid_y <- mean(new_coor$Y)
+
+  new_coor$X <- new_coor$X - mid_x
+  new_coor$Y <- new_coor$Y - mid_y
+
   # 放大/缩小
   new_coor$X <- new_coor$X * scale
   new_coor$Y <- new_coor$Y * scale * aspect_ratio
@@ -234,6 +272,9 @@ transform_coors <- function(coors, scale = 1, aspect_ratio = 1,
   # 镜像变换
   if (mirror_x) new_coor$X <- -new_coor$X
   if (mirror_y) new_coor$Y <- -new_coor$Y
+  # 还原到原始位置
+  new_coor$X <- new_coor$X + mid_x
+  new_coor$Y <- new_coor$Y + mid_y
 
   coors <- new_coor
   return(coors)
@@ -443,17 +484,17 @@ as_circle_tree <- \(){
 }
 
 big_layout <- \(zoom1, layout1, nodeGroup){
-  c_net_update(igraph::make_ring(nlevels(nodeGroup$group))) -> tmp_da_net
+  tmp_g <- igraph::make_ring(nlevels(nodeGroup$group))
+  V(tmp_g)$name <- levels(nodeGroup$group)
+  c_net_update(tmp_g) -> tmp_da_net
   da <- get_coors(layout1, tmp_da_net)
-  da$name <- NULL
-
-  if (!all(levels(nodeGroup$group) %in% rownames(da))) rownames(da) <- levels(nodeGroup$group)
+  rownames(da) <- da$name
+  da <- da[, c("X", "Y")]
 
   # center of each group
   scale_f <- (ceiling(max(da) - min(da)))
   scale_f <- ifelse(scale_f == 0, 1, scale_f / 2)
   da <- da / scale_f * zoom1
-  colnames(da) <- c("X", "Y")
   return(da)
 }
 
@@ -534,11 +575,15 @@ g_layout <- function(go, group = "module", group_order = NULL, layout1 = in_circ
   # get coors
   all_coors <- setNames(vector("list", nlevels(nodeGroup$group)), levels(nodeGroup$group))
   for (i in levels(nodeGroup$group)) {
+    # print(i)
     nodeGroup[nodeGroup[, "group"] == i, "ID"] -> tmpid
     igraph::subgraph(go, tmpid) -> tmp_net
 
     get_coors(layoutls[[i]], tmp_net, ...) -> coors
     data <- coors
+    if (nrow(data) == 1) {
+      data <- data.frame(name = data$name, X = 0, Y = 0) %>% as_coors()
+    }
     if ("igraph_layout_spec" %in% class(layoutls[[i]])) {
       data[, c("X", "Y")] <- igraph::norm_coords(as.matrix(data[, c("X", "Y")]))
     }
@@ -689,22 +734,53 @@ g_layout_multi_layer <- function(go, layout = igraph::in_circle(), group = "v_gr
   n <- length(unique(igraph::vertex.attributes(go)[[group]]))
   if (n < 2) stop("n should bigger than 1")
 
-  get_v(go) %>% dplyr::pull(!!group) -> group1
+  get_v(go) %>%
+    dplyr::pull(!!group) %>%
+    as.factor() -> group1
+  if (!is.null(group_order)) group1 <- pcutils::change_fac_lev(group1, group_order)
   group1_level <- table(group1)
   `_internal_group` <- NULL
   V(go)$`_internal_group` <- group1
 
+  # layout of vertexes in one group
+  {
+    layoutls <- list()
+    if (is_layout(layout)) {
+      layoutls <- rep(list(layout), nlevels(group1))
+    } else if (all(class(layout) == "list")) {
+      if (is.null(names(layout))) {
+        layoutls <- rep(layout, len = nlevels(group1))
+      } else {
+        for (i in levels(group1)) {
+          if (i %in% names(layout)) {
+            layoutls[[i]] <- layout[[i]]
+          } else {
+            layoutls[[i]] <- igraph::in_circle()
+            warning("layout of ", i, " not set, use in_circle()")
+          }
+        }
+      }
+    }
+    if (is.null(names(layoutls))) names(layoutls) <- levels(group1)
+  }
+
   layout2_ls <- list()
   for (i in names(group1_level)) {
     c_net_filter(go, `_internal_group` == i) %>%
-      c_net_layout(method = layout, order_by = group2, order_ls = group2_order) %>%
+      c_net_layout(method = layoutls[[i]], order_by = group2, order_ls = group2_order) %>%
       transform_coors(shear_x = 3, aspect_ratio = 0.2) -> layout2_ls[[i]]
   }
 
+  if (scale_node_num) {
+    zoom2 <- group1_level / mean(group1_level)
+  } else {
+    zoom2 <- rep(1, nlevels(group1))
+  }
+
   g_layout(go,
     group_order = group_order,
     group = group, layout1 = data.frame(X = 0, Y = seq(-1, 1, length = length(group1_level))),
-    zoom1 = 1, zoom2 = group1_level / mean(group1_level),
+    zoom1 = 1, zoom2 = zoom2,
     layout2 = layout2_ls
   ) -> oridata
   oridata
@@ -716,6 +792,7 @@ g_layout_multi_layer <- function(go, layout = igraph::in_circle(), group = "v_gr
 #' @param group group name (default: module)
 #' @param mode circlepack, treemap, backbone, stress
 #' @param ... add
+#' @param group_zoom zoom for each group
 #'
 #' @export
 #' @return coors
@@ -729,7 +806,7 @@ g_layout_multi_layer <- function(go, layout = igraph::in_circle(), group = "v_gr
 #'   plot(co_net_modu, coors = g_layout_nice(co_net_modu, group = "module", mode = "treemap"))
 #' }
 #' }
-g_layout_nice <- function(go, group = "module", mode = "circlepack", ...) {
+g_layout_nice <- function(go, group = "module", mode = "circlepack", group_zoom = 1, ...) {
   name <- x <- y <- NULL
   lib_ps("ggraph", library = FALSE)
   stopifnot(is_igraph(go))
@@ -752,10 +829,25 @@ g_layout_nice <- function(go, group = "module", mode = "circlepack", ...) {
   colnames(coors) <- c("name", "X", "Y")
   coors <- as_coors(coors)
 
+  for (i in unique(nodeGroup$group)) {
+    tmp_id <- nodeGroup[nodeGroup[, "group"] == i, "ID"]
+    transform_coors(coors[coors$name %in% tmp_id, ], scale = group_zoom) -> coors[coors$name %in% tmp_id, ]
+  }
+
   coors <- rescale_coors(coors)
   return(coors)
 }
 
+#' @rdname g_layout_nice
+#' @export
+get_big_lay_nice <- function(go, group = "module", mode = "circlepack", ...) {
+  coors <- g_layout_nice(go, group, mode, ...)
+  # 提取circlepack的分组坐标
+  big_lay <- coors[grepl("group_", coors$name), ]
+  big_lay$name <- gsub("group_", "", big_lay$name)
+  return(big_lay)
+}
+
 #' @rdname g_layout_nice
 #' @export
 g_layout_circlepack <- \(go, group = "module", ...){
@@ -793,6 +885,7 @@ g_layout_stress <- \(go, group = "module", ...){
 #' @param order_ls manual the discrete variable with a vector, or continuous variable with "desc" to decreasing
 #' @param seed random seed
 #' @param rescale rescale the coordinates to (0,1)
+#' @param order_circle order nodes from the center of a circle
 #'
 #' @return A coors object (data.frame with class "coors" and attribute "curved")
 #' @export
@@ -821,7 +914,8 @@ spatstat_layout <- function(go, win,
                             type = c("random", "regular"),
                             mode = c("surface", "boundary"),
                             jitter = 0,
-                            curved = NULL, order_by = NULL, order_ls = NULL,
+                            curved = NULL,
+                            order_by = NULL, order_ls = NULL, order_circle = FALSE,
                             seed = 1234, rescale = TRUE) {
   lib_ps("spatstat.geom", library = FALSE)
   lib_ps("spatstat.random", library = FALSE)
@@ -851,9 +945,14 @@ spatstat_layout <- function(go, win,
     pts$y <- c(pts$y, attributes(pts)$rejects$y)
   }
 
-  coors <- data.frame(X = pts$x, Y = pts$y) %>%
-    dplyr::arrange_all() %>%
-    as.matrix()
+  if (!order_circle) {
+    coors <- data.frame(X = pts$x, Y = pts$y) %>%
+      dplyr::arrange_all() %>%
+      as.matrix()
+  } else {
+    coors <- order_circle_func(as.matrix(data.frame(X = pts$x, Y = pts$y)))
+  }
+
   # order
   get_v(go) -> tmp_v
   coors <- order_tmp_v_name(tmp_v, coors, order_by = order_by, order_ls = order_ls)
