@@ -58,7 +58,7 @@ plot.metanet <- function(x, ...) {
 }
 
 get_net_main <- function(n_index) {
-  main <- "Network"
+  main <- ""
   if (!is.null(n_index$n_type)) {
     switch(n_index$n_type,
       "single" = {
@@ -80,7 +80,7 @@ get_net_main <- function(n_index) {
         main <- "Venn network"
       },
       default = {
-        main <- "Network"
+        main <- ""
       }
     )
   }
@@ -88,7 +88,7 @@ get_net_main <- function(n_index) {
 }
 
 scale_size_width <- function(tmp_v, tmp_e, vertex_size_range, edge_width_range) {
-  {        v_groups <- unique(tmp_v$v_group)
+  {    v_groups <- unique(tmp_v$v_group)
     nice_size <- ceiling(100 / sqrt(nrow(tmp_v))) + 1
 
     vertex_size_range_default <- rep(list(c(max(nice_size * 0.4, 3), min(nice_size * 1.6, 12))), length(v_groups))
@@ -128,6 +128,8 @@ scale_size_width <- function(tmp_v, tmp_e, vertex_size_range, edge_width_range)
   envir <- parent.frame()
   assign("node_size_text", node_size_text, envir)
   assign("edge_width_text", edge_width_text, envir)
+  assign("vertex_size_range", vertex_size_range, envir)
+  assign("edge_width_range", edge_width_range, envir)
   assign("tmp_e", tmp_e, envir)
   assign("tmp_v", tmp_v, envir)
 }
@@ -231,8 +233,12 @@ get_show_labels <- function(tmp_v, labels_num) {
     }
   }
   {
-    if (labels_num == "all") {
-      tmp_v %>% dplyr::pull(name) -> toplabel
+    if (is.character(labels_num)) {
+      if (identical(labels_num, "all")) {
+        tmp_v %>% dplyr::pull(name) -> toplabel
+      } else {
+        intersect(tmp_v %>% dplyr::pull(name), labels_num) -> toplabel
+      }
     } else {
       if (labels_num >= 1) {
         tmp_v %>%
@@ -304,6 +310,7 @@ produce_c_net_legends <- function(tmp_v, tmp_e, vertex_frame_width,
                                   pie_legend, pie_legend_title, pie_legend_order, pie_color,
                                   ...) {
   color <- e_type <- lty <- e_class <- v_class <- shape <- left_leg_x <- right_leg_x <- NULL
+  v_highlight <- v_highlight_from <- v_highlight_to <- NULL
 
   legend_position_default <- c(left_leg_x = -2, left_leg_y = 1, right_leg_x = 1.2, right_leg_y = 1)
 
@@ -337,6 +344,9 @@ produce_c_net_legends <- function(tmp_v, tmp_e, vertex_frame_width,
       )
     }
 
+    if ("v_highlight" %in% colnames(tmp_v)) {
+      tmp_v <- dplyr::arrange(tmp_v, -v_highlight)
+    }
     for (g_i in vgroups) {
       if ("count" %in% names(tmp_v)) {
         tmp_v1 <- tmp_v[tmp_v$v_group == g_i, c("v_class", "color", "shape", "count")]
@@ -358,7 +368,7 @@ produce_c_net_legends <- function(tmp_v, tmp_e, vertex_frame_width,
         eee <- table(tmp_v1$v_class)
         if (!is.null(attributes(tmp_v)$skeleton)) {
           eee <- setNames(tmp_v1$count, tmp_v1$v_class)
-          legend_number <- FALSE
+          # legend_number <- FALSE
         }
         le_text <- paste(vclass, eee[vclass], sep = ": ")
       } else {
@@ -428,7 +438,15 @@ produce_c_net_legends <- function(tmp_v, tmp_e, vertex_frame_width,
     right_leg_y <- right_leg_y - 0.5 * legend_cex
   }
 
+  if (nrow(tmp_e) == 0) {
+    return(invisible())
+  }
+
   if (edge_legend) {
+    if ("v_highlight_from" %in% colnames(tmp_e)) {
+      tmp_e <- dplyr::arrange(tmp_e, -v_highlight_from, -v_highlight_to)
+    }
+
     tmp_e$e_type <- factor(tmp_e$e_type, levels = custom_sort(unique(tmp_e$e_type)))
     edges <- pcutils::change_fac_lev(tmp_e$e_type, edge_legend_order)
     edges <- levels(edges)
@@ -531,6 +549,9 @@ produce_c_net_legends <- function(tmp_v, tmp_e, vertex_frame_width,
 #' @param seed random seed, default:1234, make sure each plot is the same.
 #' @param params_list a list of parameters, e.g. list(edge_legend = TRUE, lty_legend = FALSE), when the parameter is duplicated, the format argument will be used rather than the argument in params_list.
 #' @param rescale Logical constant, whether to rescale the coordinates to the (-1,1).
+#' @param arrow_size_cex arrow size cex, default 1, relative to the vertex size
+#' @param arrow_width_cex arrow width cex, default 1, relative to the vertex size
+#' @param label_cex label cex, default 1, relative to the vertex size
 #'
 #' @family plot
 #' @return a network plot
@@ -558,6 +579,7 @@ c_net_plot <- function(go, coors = NULL, ..., labels_num = NULL,
                        lty_legend = FALSE, lty_legend_title = "Edge class", lty_legend_order = NULL,
                        module_legend = FALSE, module_legend_title = "Module", module_legend_order = NULL,
                        pie_legend = FALSE, pie_legend_title = "Pie part", pie_legend_order = NULL,
+                       label_cex = 1, arrow_size_cex = 1, arrow_width_cex = 1,
                        params_list = NULL, rescale = FALSE,
                        seed = 1234) {
   if (!is.null(params_list)) {
@@ -647,14 +669,15 @@ c_net_plot <- function(go, coors = NULL, ..., labels_num = NULL,
       vertex.pie = pie_value_list,
       vertex.pie.color = pie_color_list,
       ...,
+      vertex.size2 = tmp_v$size / 2,
       vertex.frame.width = 0.5,
       main = main,
       rescale = rescale,
       vertex.label.font = 1,
-      vertex.label.cex = 0.07 * tmp_v$size,
+      vertex.label.cex = label_cex * 0.05 * tmp_v$size,
       vertex.label = tmp_v$label,
-      edge.arrow.size = 0.3 * tmp_e$width * 3,
-      edge.arrow.width = 0.6 * tmp_e$width * 3,
+      edge.arrow.size = arrow_size_cex * 0.2 * tmp_e$width,
+      edge.arrow.width = arrow_width_cex * 1 * tmp_e$width,
       edge.curved = edge_curved,
       margin = c(0, 0, 0, 0)
     )
